@@ -24,7 +24,6 @@ import (
 	"reflect"
 	"sync"
 	"testing"
-	"time"
 
 	"github.com/davecgh/go-spew/spew"
 	"github.com/ethereum/go-ethereum/common"
@@ -38,11 +37,7 @@ func init() {
 
 // shared test variables
 var (
-	futureExp          = uint64(time.Now().Add(10 * time.Hour).Unix())
-	testTarget         = NodeID{0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1}
-	testRemote         = rpcEndpoint{IP: net.ParseIP("1.1.1.1").To4(), UDP: 1, TCP: 2}
-	testLocalAnnounced = rpcEndpoint{IP: net.ParseIP("2.2.2.2").To4(), UDP: 3, TCP: 4}
-	testLocal          = rpcEndpoint{IP: net.ParseIP("3.3.3.3").To4(), UDP: 5, TCP: 6}
+	testLocal = rpcEndpoint{IP: net.ParseIP("3.3.3.3").To4(), UDP: 5, TCP: 6}
 )
 
 // type udpTest struct {
@@ -126,7 +121,7 @@ var (
 // 	defer test.table.Close()
 //
 // 	// put a few nodes into the table. their exact
-// 	// distribution shouldn't matter much, altough we need to
+// 	// distribution shouldn't matter much, although we need to
 // 	// take care not to overflow any bucket.
 // 	targetHash := crypto.Keccak256Hash(testTarget[:])
 // 	nodes := &nodesByDistance{target: targetHash}
