@@ -21,26 +21,26 @@ import (
 )
 
 type Control struct {
-	api  *Api
+	api  *API
 	hive *network.Hive
 }
 
-func NewControl(api *Api, hive *network.Hive) *Control {
+func NewControl(api *API, hive *network.Hive) *Control {
 	return &Control{api, hive}
 }
 
-func (self *Control) BlockNetworkRead(on bool) {
-	self.hive.BlockNetworkRead(on)
-}
-
-func (self *Control) SyncEnabled(on bool) {
-	self.hive.SyncEnabled(on)
-}
-
-func (self *Control) SwapEnabled(on bool) {
-	self.hive.SwapEnabled(on)
-}
-
-func (self *Control) Hive() string {
-	return self.hive.String()
+//func (self *Control) BlockNetworkRead(on bool) {
+//	self.hive.BlockNetworkRead(on)
+//}
+//
+//func (self *Control) SyncEnabled(on bool) {
+//	self.hive.SyncEnabled(on)
+//}
+//
+//func (self *Control) SwapEnabled(on bool) {
+//	self.hive.SwapEnabled(on)
+//}
+//
+func (c *Control) Hive() string {
+	return c.hive.String()
 }
